/*
 * Simulator of MCS51 (s51.cc)
 *
 * Copyright (C) 1997,97 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of MCS51 simulator: s51.

S51 is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

S51 is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with S51; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>
#include <sys/stat.h>
#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif
#include "i_string.h"

#include "simcl.h"

#include "globals.h"
#include "cmd.h"
#include "uc51cl.h"
#include "uc52cl.h"
#include "uc51rcl.h"
#include "uc251cl.h"


class cl_sim *simulator;


/*
 * Print out usage
 *____________________________________________________________________________
 *
 */

void
PrintHelp(char *name)
{
  printf("%s: %s\n", name, VERSIONSTR);
  printf("Usage: %s [-hHVvP] [-p prompt] [-t CPU] [-X freq[k|M]] "
	 "[-c file] [-s file] "
	 "[files...]\n", name);
  printf("Options:\n");
  printf("  -t CPU       Type of CPU: 51, C52, 251, etc.\n");
  printf("  -X freq[k|M] XTAL frequency\n");
  printf("  -c file      Open command console on `file'\n");
  printf("  -s file      Connect serial interface to `file'\n");
  printf("  -p prompt    Specify string for prompt\n");
  printf("  -P           Promp is a null ('\\0') character\n");
  printf("  -V           Verbose mode\n");
  printf("  -v           Print out version number\n");
  printf("  -H           Print out types of known CPUs\n");
  printf("  -h           Print out this help\n");
}


/*
 * Simulator: makes CPU object, reads input files and calls command interpreter
 *_____________________________________________________________________________
 *
 */

int
do_s51(class cl_sim *sim, int argc, char *argv[])
{
  char *dis;
  int index, i;

  sim->cmd_in= cmd_conin;
  sim->cmd_out= cmd_conout;
  i= fprintf((cmd_conout==NULL)?(stdout):(cmd_conout),
	     "S51 %s, Copyright (C) 1997 Daniel Drotos, Talker Bt.\n"
	     "S51 comes with ABSOLUTELY NO WARRANTY; for details type "
	     "`show w'.\n"
	     "This is free software, and you are welcome to redistribute it\n"
	     "under certain conditions; type `show c' for details.\n",
	     VERSIONSTR);
  for (index= optind; index < argc; index++)
    {
      long l;

      if ((l= ((class t_uc51*)(sim->uc))->read_hex_file(argv[index])) >= 0)
	fprintf((cmd_conout==NULL)?(stdout):(cmd_conout),
		"%ld bytes read from %s\n", l, argv[index]);
    }

  ((class t_uc51*)(sim->uc))->do_cmd();

  //delete uc;
  return(0);
}

class cl_sim51: public cl_sim
{
public:
  cl_sim51(char *more_args, int iargc, char *iargv[]):
    cl_sim(more_args, iargc, iargv) {}

  virtual int proc_arg(char optopt, char *optarg)
    {
      char *cpu_type= NULL, *cp;
      int i;

      switch (optopt)
	{

	  /*      case 'd':
		  strncpy(_example_char, optarg, 127);
		  break;*/

	case 't':

	  if (cpu_type)
	    free(cpu_type);
	  cpu_type= strdup(optarg);
	  for (cp= cpu_type; *cp; *cp= toupper(*cp), cp++);
	  break;

	case 'X':

	  for (cp= optarg; *cp; *cp= toupper(*cp), cp++);
	  XTAL= strtod(optarg, &cp);
	  if (*cp == 'K')
	    XTAL*= 1e3;
	  if (*cp == 'M')
	    XTAL*= 1e6;
	  break;

	case 'c':

	  if ((cmd_conin= fopen(optarg, "r+")) == NULL)
	    {
	      fprintf(stderr,
		      "Can't open `%s': %s\n", optarg, strerror(errno));
	      return(3);
	    }
	  else
	    cmd_conout= cmd_conin;
	  break;

	case 's':

	  if (Ser_in)
	    fclose(Ser_in);
	  if ((Ser_in= fopen(optarg, "r")) == NULL)
	    {
	      fprintf(stderr,
		      "Can't open `%s': %s\n", optarg, strerror(errno));
	      return(4);
	    }
	  if (Ser_out)
	    fclose(Ser_out);
	  if ((Ser_out= fopen(optarg, "w")) == NULL)
	    {
	      fprintf(stderr,
		      "Can't open `%s': %s\n", optarg, strerror(errno));
	      return(4);
	    }
	  break;

	case 'S':

	  cp= (char *)malloc(strlen(optarg)+2);
	  strcpy(cp, optarg);
	  strcat(cp, "i");
	  if (mkfifo(cp, 0600) < 0)
	    {
	      fprintf(stderr, "Can't create `%si': %s\n",
		      optarg, strerror(errno));
	      return(4);
	    }
	  if (Ser_in)
	    fclose(Ser_in);
	  if ((Ser_in= fopen(cp, "r")) == NULL)
	    {
	      fprintf(stderr,
		      "Can't open `%s': %s\n", optarg, strerror(errno));
	      return(4);
	    }
	  cp[strlen(cp)-1]= 'o';
	  if (mkfifo(cp, 0600) < 0)
	    {
	      fprintf(stderr, "Can't create `%so': %s\n",
		      optarg, strerror(errno));
	      return(4);
	    }
	  if (Ser_out)
	    fclose(Ser_out);
	  if ((Ser_out= fopen(cp, "w")) == NULL)
	    {
	      fprintf(stderr,
		      "Can't open `%s': %s\n", optarg, strerror(errno));
	      return(4);
	    }
	  free(cp);
	  break;
	  
	case 'p':
	  
	  if (prompt_str)
	    free(prompt_str);
	  prompt_str= strdup(optarg);
	  break;

	case 'P':

	  if (prompt_str)
	    free(prompt_str);
	  prompt_str= NULL;
	  break;

	case 'h':

	  PrintHelp("s51");
	  exit(0);
	  break;
	
	case 'H':
	  i= 0;
	  while (cpus[i].type_str != NULL)
	    {
	      printf("%s\n", cpus[i].type_str);
	      i++;
	    }
	  exit(0);
	  break;

	case '?':

	  if (isprint(optopt))
	    fprintf(stderr, "Unknown option `-%c'.\n", optopt);
	  else
	    fprintf(stderr, "Unknown option character `\\x%x'.\n", optopt);
	  return(1);
	  break;

	dafault:
	  // should never happen...
	  abort();
	}
      return(0);
    }

  virtual class cl_uc *mk_controller(/*char *cpu_type, class cl_sim *sim*/void)
    {
      int i;

      i= 0;
      while ((cpus[i].type_str != NULL) &&
	     (strcmp(get_sarg('t'), cpus[i].type_str) != 0))
	i++;
      if (cpus[i].type_str == NULL)
	return(NULL);
      switch (cpus[i].type)
	{
	case CPU_51: case CPU_31:
	  return(new t_uc51(cpus[i].type, cpus[i].technology,
			    Ser_in, Ser_out, Verbose, this));
	case CPU_52: case CPU_32:
	  return(new t_uc52(cpus[i].type, cpus[i].technology,
			    Ser_in, Ser_out, Verbose, this));
	case CPU_51R:
	  return(new t_uc51r(cpus[i].type, cpus[i].technology,
			     Ser_in, Ser_out, Verbose, this));
	case CPU_251:
	  return(new t_uc251(cpus[i].type, cpus[i].technology,
			     Ser_in, Ser_out, Verbose, this));
	}
      return(NULL);
    }
};

/*
 * Main function of the Simulator of MCS51. Everything starts here.
 *___________________________________________________________________________
 *
 */

int
main(int argc, char *argv[])
{
  int  retval;

  simulator= new cl_sim51("t:X:c:s:S:p:PhH", argc, argv);
  prompt_str= strdup("> ");
  if (simulator->get_sarg('t') == NULL)
    {
      simulator->arguments->add(new cl_prg_arg('t', 0, 0, "C51"));
    }
  simulator->init();
  retval= do_s51(simulator, argc, argv);
  delete simulator;

  return(retval);
}

/* End of s51.cc */
